import 'dart:math' as math;

import 'package:grizzly/src/utils/comap.dart';

extension DoubleExt on double {
  double toPrecision(int places) {
    num pld = math.pow(10,places);
    return (this * pld).truncateToDouble()/pld;
  }
}

extension DoubleIterableExt on Iterable<double> {
  Iterable<double> plus(/* Iterable<num> | num */ other) {
    if (other is num) {
      return map((e) => e + other);
    } else if (other is Iterable<num>) {
      return comap<double, num?, num?>(
          this, other, (a, b) => (a ?? 0).toDouble() + (b ?? 0));
    }

    throw UnsupportedError('only num and Iterable<num> are supported');
  }

  Iterable<double> operator -(/* Iterable<num> | num */ other) {
    if (other is num) {
      return map((e) => e - other);
    } else if (other is Iterable<num>) {
      return comap<double, num?, num?>(
          this, other, (a, b) => (a ?? 0).toDouble() - (b ?? 0));
    }

    throw UnsupportedError('only num and Iterable<num> are supported');
  }

  Iterable<double> operator *(/* Iterable<num> | num */ other) {
    if (other is num) {
      return map((e) => e * other);
    } else if (other is Iterable<num>) {
      return comap<double, num?, num?>(
          this, other, (a, b) => (a ?? 0).toDouble() * (b ?? 0));
    }

    throw UnsupportedError('only num and Iterable<num> are supported');
  }

  Iterable<double> operator /(/* Iterable<num> | num */ other) {
    if (other is num) {
      return map((e) => e / other);
    } else if (other is Iterable<num>) {
      return comap<double, num?, num?>(
          this, other, (a, b) => (a ?? 0).toDouble() / (b ?? 0));
    }

    throw UnsupportedError('only num and Iterable<num> are supported');
  }

  Iterable<int> operator ~/(/* Iterable<num> | num */ other) {
    if (other is num) {
      return map((e) => e ~/ other);
    } else if (other is Iterable<num>) {
      return comap<int, num?, num?>(
          this, other, (a, b) => (a ?? 0) ~/ (b ?? 0));
    }

    throw UnsupportedError('only num | Iterable<num> are supported');
  }

  double dot([Iterable<num?>? other]) => cofold<double, double, num?>(
      0, this, other ?? this, (value, double? a, num? b) => value + a! * (b ?? 0));

  bool isClose(Iterable<num> v, {double tol = 1e-8}) {
    if (length != v.length) return false;
    for (int i = 0; i < length; i++) {
      if ((elementAt(i) - v.elementAt(i)).abs() > tol) return false;
    }
    return true;
  }

  bool isCloseScalar(num v, {double tol = 1e-8}) {
    for (int i = 0; i < length; i++) {
      if ((elementAt(i) - v).abs() > tol) return false;
    }
    return true;
  }
}
